part of tecfy_core_client_package;

class EntityListController extends BaseController {
  EntityService _service;
  Entity entity;
  List<dynamic> data;
  List<dynamic> filterdData;
  List<EntityFilter> filters;
  List<Property> searchableProperties;
  int pageIndex = 0;
  int pageIndexMax = 0;
  bool hasNextPage = true;

  TextEditingController searchController;
  RefreshController refreshController =
      RefreshController(initialRefresh: false); // for pull to refrsh

  EntityListController(State<StatefulWidget> state, String entityName)
      : super(state) {
    refreshController.footerMode.addListener(footerListner);
    searchController = TextEditingController();
    _service = EntityService();
    filters = List.empty(growable: true); // List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    searchableProperties =
        entity.properties.where((e) => e.isSearchable).toList();

    loadData();
  }

  @override
  dispose() {
    refreshController?.dispose();
    searchController?.dispose();
    super.dispose();
  }

  String sortProp;
  bool sortAsc = true;

  addChildren(items, List<dynamic> rows) {
    for (var item in items) {
      rows.add(item);
      if (item['_expanded'] == true &&
          item['_children'] != null &&
          item['_children'].length > 0) {
        rows = addChildren(item['_children'], rows);
      }
    }
    return rows;
  }

  globalSearchText() {
    if (searchController.text.isEmpty) {
      filterdData = data;
    } else {
      filterdData = data.where((d) {
        return searchableProperties
            .any((p) => d[p.dbName].toString().contains(searchController.text));
      }).toList();
    }

    if (entity.isTree) {
      filterdData = addChildren(filterdData, List<dynamic>());
    }
    setState();
  }

  sortBy(Property prop) {
    if (prop.dbName != sortProp) {
      sortProp = prop.dbName;
      sortAsc = true;
    } else
      sortAsc = !sortAsc;

    data.sort((a, b) => sortAsc
        ? a[sortProp].toString().compareTo(b[sortProp].toString())
        : b[sortProp].toString().compareTo(a[sortProp].toString()));
    globalSearchText();
    setState();
  }

  bool get hasPageing => (data?.length ?? 0) >= EntityService.pageSize;

  footerListner() async {
    print('==========on load ====== ${refreshController.footerStatus}');
    if (refreshController.footerStatus != LoadStatus.loading) return;
    var success = await gotoMextPageIndex();
    if (success && hasNextPage) {
      refreshController.loadComplete();
    } else if (success && !hasNextPage) {
      //refreshController.loadComplete();
      refreshController.loadNoData();
    } else {
      refreshController.loadFailed();
    }
  }

  Future<bool> gotoMextPageIndex() {
    pageIndex++;
    pageIndexMax = max(pageIndexMax, pageIndex);
    return loadData(loadFromServer: true, withLoading: false);
  }

  gotoPageIndex(int i) {
    pageIndex = i;
    setLoading();
    loadData(loadFromServer: true);
  }

  var isLoading = false;
  Future<bool> loadData(
      {bool loadFromServer = false, bool withLoading = true}) async {
    if (isLoading) return true;
    isLoading = true;
    var result = await _service.getData(entity,
        filter: filters,
        forceFromServer: loadFromServer,
        pageIndex: pageIndex,
        filterParentId: entity.isTree);
    if (withLoading) setResume();
    if (!result.success) {
      data = [];
      if (withLoading) {
        setState();
        result.showMessage(context);
      }
      globalSearchText();
      isLoading = false;
      return false;
    } else {
      if (LayoutService.isMobile(context) && pageIndex > 0 && !entity.isLocal)
        data.addAll(result.data);
      else
        data = result.data;

      if (result.data.length < EntityService.pageSize) hasNextPage = false;
    }
    globalSearchText();
    // if(pageIndex == 0) refreshAnimation();
    isLoading = false;
    return true;
  }

  Future loadFromServer() async {
    await loadData(loadFromServer: true);
    refreshController.refreshCompleted();
  }

  Future onRefresh() async {
    pageIndex = 0;
    pageIndexMax = 0;
    hasNextPage = true;
    setLoading();
    await loadData(loadFromServer: true);
  }

  Future onAdd() async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onUpdate(d) async {
    var result = await AppNavigator.navigateTo(
        context, () => EntityFormPage(entity.dbName, d),
        animationType: LayoutService.isMobile(context)
            ? AnimationTypes.slideUp
            : AnimationTypes.opacity);
    if (result == true) {
      loadFromServer();
    }
  }

  Future onDelete(d) async {
    if (await confirmMessage('هل انت متاكد من الحذف؟')) {
      var result = await EntityService().delete(entity, d['id']);
      if (result.success) {
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        loadFromServer();
      } else {
        showMessage('خطأ اثناء الحذف', type: MessageType.error);
      }
    }
  }

  expandItem(Map d) async {
    try {
      if (!entity.isTree) return true;
      if (d['_children'] != null) {
        d['_expanded'] = !d['_expanded'];
        globalSearchText();
        return;
      }

      if (isLoading) return true;
      isLoading = true;
      print('expand ${d['id']}');
      setLoading();
      var result = await _service.getData(entity,
          filter: filters,
          forceFromServer: true,
          pageIndex: pageIndex,
          filterParentId: true,
          parentId: d['id'].toString());

      setResume();
      isLoading = false;
      if (!result.success) {
        //data = [];
        return;
      }
      for (var r in result.data) {
        if (d['_tree_level'] != null)
          r['_tree_level'] = d['_tree_level'] + 1;
        else
          r['_tree_level'] = 1;
      }
      d['_children'] = result.data;
      d['_expanded'] = true;
      // data.insertAll(
      //     data.indexWhere((i) => i['id'] == d['id']) + 1, result.data);
      globalSearchText();
    } catch (err) {
      print(err);
      setResume();
      isLoading = false;
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName =>
      '${entity.properties.firstWhere((d) => d.isImage).dbName}Id';
}
