part of tecfy_core_client_package;

class EntityFilterPage extends StatefulWidget {

  final String entityName;
  final List<Property> properties;
  final List<EntityFilter> filters;
  EntityFilterPage(this.entityName,this.properties,[this.filters]);
  @override
  _EntityFilterPageState createState() => _EntityFilterPageState();
}

class _EntityFilterPageState extends AppStateView<EntityFilterPage,EntityFilterController> {
  
  List<EntityFilter> filters;
  @override
  void initState() {
    _controller = EntityFilterController(this,widget.entityName,widget.properties);
    filters = widget.filters ?? List<EntityFilter>();
    if(filters.length == 0) filters.add(EntityFilter());
    super.initState();
  }

  onCancel(){
    Navigator.pop(context,null);
  }

  onFilter(){
    Navigator.pop(context,filters);
  }

  onAdd(){
    setState(() {
     filters.add(EntityFilter()) ;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: MainAppBar(context: context,title: '${_controller.entityName} (${AppLocale.translate(context, 'Filter')})', actions: <Widget>[
        IconButton(icon:Icon(Icons.close),onPressed: onCancel)
      ],),
      body: ListView(
        children: filters.map((filter)=>getCard(filter)).toList()
          ..add(FlatButton.icon(icon:Icon(Icons.add),onPressed: onAdd, label: Text(AppLocale.translate(context,'Add Row'))))
          ..add(FlatButton.icon(icon:Icon(Icons.search),onPressed: onFilter, label: Text(AppLocale.translate(context,'OK')))),
      ),
      // body: Table(
      //   children: filters.map((filter)=>TableRow(
      //     children: [
      //       TableCell(child: propertyList(filter)),
      //       TableCell(child: operatorList(filter)),
      //       TableCell(child: valueWidget(filter)),
      //     ]
      //   )).toList()
      //   ..insert(0,TableRow(
      //     children: [
      //       TableCell(child: Text('Property')),
      //       TableCell(child: Text('Operator')),
      //       TableCell(child: Text('Value')),
      //     ]
      //   )),
      // ),
    );
  }

  Widget getCard(EntityFilter filter){
    return Card(
      margin: EdgeInsets.all(10),
      borderOnForeground: true,
      color: Colors.lightBlue[100],
      child: Column(
        children: <Widget>[
          Align(alignment: Alignment.topRight,child:  IconButton(icon: Icon(Icons.close, color: Colors.red,),onPressed: (){
            filters.remove(filter);
            _controller.setState();
          })),
          Row(
            children: <Widget>[
              SizedBox(width: 10),
              Expanded(child: propertyList(filter),),
              operatorList(filter)
            ],
          ),
          Padding(child: valueWidget(filter), padding: EdgeInsets.only(top: 0,bottom: 0,right:10, left:10))
        ],
      ),
    );
  }

  Widget propertyList(EntityFilter filter){
    return DropdownButtonHideUnderline(child: DropdownButton(
      onChanged: (v){
        filter.property = v;
        filter.value = null;
        
        setState(() {});
      },
      value: filter.property,
      items: _controller.properties.map((p)=> 
        DropdownMenuItem(child: Text(p.nameAr),value: p)
      ).toList(),
    ));
  }

  DropdownButton operatorList(EntityFilter filter){
    return DropdownButton(
      onChanged: (v){
        filter.operatorId = v;
        setState(() {});
      },
      value: filter.operatorId,
      items: [
        DropdownMenuItem(child: Text('=='),value:'E'),
        DropdownMenuItem(child: Text('>='),value:'GE'),
        DropdownMenuItem(child: Text(AppLocale.translate(context,'Contain')),value:'C'),
        DropdownMenuItem(child: Text(AppLocale.translate(context,'Not Contain')),value:'NC'),
      ],
    );
  }

  Widget valueWidget(EntityFilter filter){
    if(filter.property == null) return Container();
    return PorpertyComponent(property: filter.property,onChange: (v){
      setState(() => filter.value =v);
    },
    data:{ filter.property.dbName: filter.value});
  }

}