part of tecfy_core_client_package;

class EntityFormPage extends StatefulWidget {
  final String entity;
  final dynamic data;
  EntityFormPage(this.entity, [this.data]);
  @override
  _EntityFormPageState createState() => _EntityFormPageState();
}

class _EntityFormPageState extends AppStateView<EntityFormPage,EntityFormController> {
  
  @override
  void initState() {
    _controller = EntityFormController(this, widget.entity, widget.data);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return DefaultTabController(
        length: _controller.tabs.keys.length,
        child: BasePage(context, _controller.entity.name,
            appBarActions: !_controller.canSave
                ? []
                : <Widget>[
                  LayoutService.isMobile(context) ? 
                    IconButton(
                        icon: Icon(Icons.save, color: ThemeService.mainColor),
                        //color: ThemeService.secondryColor,
                        tooltip: trans('Save'),
                        onPressed: _controller.onSave) :
                    AppButtonIconText(
                        icon: Icon(Icons.save, color: ThemeService.textOnMainColor),
                        color: ThemeService.secondryColor,
                        label: trans('Save'),
                        onPressed: _controller.onSave)
                  ],
            appBarBottom: _controller.tabs.length > 1
                ? (LayoutService.isMobile(context)
                    ? _getTabBar()
                    : Align(
                        alignment: LocalConfigService.isArabic ?  Alignment.centerRight : Alignment.centerLeft,
                        child: Container(
                            child: _getTabBar(),
                            constraints: BoxConstraints(
                                maxWidth: (_controller.tabs.length * 150)
                                    .toDouble()))))
                : null,
            child: Align(
                alignment: Alignment.topCenter,
                child: Container(
                  constraints:
                      BoxConstraints(maxWidth: ThemeService.containerWidth),
                  child: _controller.tabs.length > 1
                      ? TabBarView(
                          children: _controller.tabs.keys
                              .map<Widget>(
                                  (tab) => _getData(_controller.tabs[tab]))
                              .toList())
                      : _getData(_controller.properties),
                ))));
  }

  _getTabBar() {
    return TabBar(
        labelColor: ThemeService.mainColor,
        tabs: _controller.tabs.keys
            .map((tab) => Tab(child: Text(tab ?? '')))
            .toList());
  }

  _getData(List<Property> props) {
    return 
    props.length == 1 || LayoutService.isMobile(context)
        ? ListView(
          padding: props.length == 1 ? null : EdgeInsets.symmetric(horizontal: 20),
          children: props
                .map<Widget>((p) =>
            PorpertyComponent(
              readOnly: _controller.readOnly,
              property: p,
              entity: _controller.entity,
              //filterValue: _controller.data[props.first.filterValueField],
              data: _controller.data,
              onChange: (v) => _controller.onValueChanged(
                  v,
                  p.dbName), //_controller.data[props.first.dbName] = v,
            )
          ).toList()
          )
        : 
        SingleChildScrollView(child:Wrap(
           alignment: WrapAlignment.start,
            children: props
                .map<Widget>((p) =>
                Container(
                  width: p.fullWidth ? null : _getWidth(),
                  margin: EdgeInsets.symmetric(horizontal: 10),
                  child:
                 PorpertyComponent(
                      readOnly: _controller.readOnly,
                      property: p,
                      entity: _controller.entity,
                      // onChange: (v) => _controller.data[p.dbName] = v,
                      //filterValue: _controller.data[p.filterValueField],
                      data: _controller.data,
                      onChange: (v) => _controller.onValueChanged(v, p.dbName),
                    )))
                .toList()));
  }

  _getWidth() {
    var fullwidth = LayoutService.getWidth(context);

    fullwidth -= (ThemeService.menuWidth + 100);
    if (fullwidth > ThemeService.containerWidth) {
      fullwidth = ThemeService.containerWidth - 50;
    }
    if (fullwidth < 600) {
      return null; // means take full width
    }
    return fullwidth / 2;
  }
}
