part of tecfy_core_client_package;

class DashboardPage extends StatefulWidget {
  @override
  _DashboardPageState createState() => _DashboardPageState();
}

class _DashboardPageState
    extends AppStateView<DashboardPage, DashboardController> {
  @override
  void initState() {
    super.initState();
    _controller = DashboardController(this);
  }

  onItemTab(type) async {
    if (type.onClick == null || type.onClick.isEmpty) return;
    if (type.onClick.startsWith('EntityList:')) {
      AppNavigator.navigateTo(context,
          () => EntityListPage(type.onClick.replaceAll('EntityList:', '')));
      return;
    }
    if (type.onClick.startsWith('Report:')) {
      var result = await ApiService.httpGetDynamic('Report/GetList');
      var reportName = type.onClick.replaceAll('Report:', '');
      AppNavigator.navigateTo(
          context,
          () => ReportDetailPage(
              result.data.firstWhere((s) => s['dbName'] == reportName), []));
      return;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      'Main Page',
      child: _controller.types == null
          ? Center(
              child: CircularProgressIndicator(
                  valueColor:
                      AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
          : Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              padding: EdgeInsets.only(top: 20),
              child: AnimationLimiter(
                child: SmartRefresher(
                  enablePullDown: true,
                  enablePullUp: false,
                  controller: _controller.refreshController,
                  header: WaterDropHeader(complete: Icon(Icons.done)),
                  onRefresh: _controller.refreshData,
                  child: Wrap(
                    alignment: WrapAlignment.center,
                    runAlignment: WrapAlignment.center,
                    crossAxisAlignment: WrapCrossAlignment.start,
                    children: List.generate(
                      _controller.types.length,
                      (index) => AnimationConfiguration.staggeredList(
                        position: index,
                        duration: const Duration(milliseconds: 375),
                        child: SlideAnimation(
                          verticalOffset: 100.0,
                          horizontalOffset: 100,
                          child: FadeInAnimation(
                              child:
                                  _controller.types[index].type == "Statistic"
                                      ? DashboardStatisticsComponenet(
                                          _controller.types[index])
                                      : _controller.types[index].type == "Chart"
                                          ? DashboardChartComponenet(
                                              _controller.types[index])
                                          : Text(_controller
                                              .types[index].displayName)),
                        ),
                      ),
                    ),
                  ),
                ),
              ),
            ),
    );
  }
}
