part of tecfy_core_client_package;

class ReportDetailPage extends StatefulWidget {
  final dynamic report;
  final List<EntityFilter> filter;
  ReportDetailPage(this.report, this.filter);
  @override
  _ReportDetailPageState createState() => _ReportDetailPageState();
}

class _ReportDetailPageState extends State<ReportDetailPage> {
  List<dynamic> data;
  dynamic totals;
  int sortColumnIndex;
  bool sortAcd = true;

  @override
  void initState() {
    getData();
    super.initState();
  }

  void onSort(String dbName, int type) {
    data.sort((a, b) => sortAcd
        ? (type == 3
            ? (a[dbName] - b[dbName]).toInt()
            : a[dbName].toString().compareTo(b[dbName].toString()))
        : (type == 3
            ? (b[dbName] - a[dbName]).toInt()
            : b[dbName].toString().compareTo(a[dbName].toString())));
    setState(() {});
  }

  getData() async {
    var result = await ApiService.httpGetDynamic(
        'Report/GetData/${widget.report['dbName']}' +
            ((widget.filter?.length ?? 0) > 0
                ? '?filter=${jsonEncode(widget.filter.map<Map>((f) => f.toJson()).toList())}'
                : ''));
    print(result.data);
    if (result.success) {
      data = result.data;
      totals = data.removeLast();
      setState(() {});
    } else {
      await AppNavigator.showMessage(context, result.message ?? "Error!");
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    print(widget.report['properties']);
    return BasePage(
        context, widget.report['name${LocalConfigService.languagePrefex}'],
        child: data == null
            ? Center(
                child: CircularProgressIndicator(
                    valueColor:
                        AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
            : SingleChildScrollView(
                child: DataTable(
                    showCheckboxColumn: true,
                    sortColumnIndex: sortColumnIndex,
                    sortAscending: sortAcd,
                    columns: widget.report['properties']
                        .map<DataColumn>((p) => DataColumn(
                              label: Text(
                                  p['name${LocalConfigService.languagePrefex}'] ??
                                      'N/A'),
                              numeric: p['type'] == 3,
                              onSort: (s, d) {
                                sortColumnIndex = s;
                                sortAcd = d;
                                onSort(p['dbName'], p['type']);
                              },
                              tooltip: p['nameEn'] ?? p['nameAr']
                            ))
                        .toList(),
                    rows: data
                        .map<DataRow>((d) => DataRow(
                              cells: widget.report['properties']
                                  .map<DataCell>((p) => getCell(p, d))
                                  .toList(),
                            ))
                        .toList()
                          ..add(DataRow(
                            cells: widget.report['properties']
                                .map<DataCell>(
                                    (p) => getCell(p, totals, totals: true))
                                .toList(),
                          ))
                    // ..insert(0, TableRow(
                    //   children: widget.report['properties'].map<TableCell>((p)=>TableCell(child: Text(p['name${LocalConfigService.languagePrefex}'] ?? 'N/A', style: ThemeService.titleStyle))).toList(),
                    // )
                    //)
                    ),
              ));
  }

  DataCell getCell(prop, data, {totals: false}) {
    var val = data[prop['dbName']];

    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return DataCell(Container());
    } else {
      if (val == null) return DataCell(Container());
    }
    switch (ValueTypes.values[prop['type'] as int]) {
      case ValueTypes.Text:
        return DataCell(Text(val,
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.Number:
        return DataCell(Text((val as num).toDouble().toStringAsFixed(2),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 12)));
      case ValueTypes.DateTime:
        return DataCell(Text(
            val.toString().replaceFirst('T', ' ').substring(0, 16),
            style: totals ? ThemeService.titleStyle : TextStyle(fontSize: 10)));
      default:
        return DataCell(Text('-'));
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}
